package banco.clientes;

/**
 * Ttulo: ArrayClientes
 * Descrio: Guarda os clientes em um array 
 *
 * @author Alexandra Barros
 * @version 1.0
 */

public class ArrayClientes implements RepositorioClientes{

  private Cliente[] clientes;
  private int indiceAtual;
  private int tamanho;

  public ArrayClientes() {
    this.tamanho = 10;
    this.indiceAtual = 0;
    this.clientes = new Cliente[tamanho];
  }

  public void inserir(Cliente cliente) throws ClienteExistenteException{
    if(!this.existe(cliente.getCPF())){
      if(this.indiceAtual == this.tamanho){
        this.tamanho = this.tamanho * 2;
        Cliente[] temp = new Cliente[this.tamanho];
        for(int i = 0; i < this.clientes.length; i++){
          temp[i] = clientes[i];
        }
        this.clientes = temp;
      }
      this.clientes[this.indiceAtual] = cliente;
      this.indiceAtual++;
    }
    else{
      throw new ClienteExistenteException("O cliente j existe.");
    }
  }

  public void remover(String cpf) throws ClienteInexistenteException{
    int i = this.procurarIndice(cpf);
    if(i!=-1){
      this.clientes[i] = this.clientes[--this.indiceAtual];
      this.clientes[this.indiceAtual] = null;
    }
    else {
      throw new ClienteInexistenteException("O cliente no existe.");
    }
  }

  public Cliente procurar(String cpf) throws ClienteInexistenteException{
    Cliente c = null;
    int i = this.procurarIndice(cpf);
    if(i!=-1){
      c = this.clientes[i];
    }
    else {
      throw new ClienteInexistenteException("O cliente no existe.");
    }
    return c;
  }

  public void atualizar(Cliente cliente) throws ClienteInexistenteException{
    int i = this.procurarIndice(cliente.getCPF());
    if(i!=-1){
      this.clientes[i] = cliente;
    }
    else {
      throw new ClienteInexistenteException("O cliente no existe.");
    }
  }

  public boolean existe(String cpf){
    int i = this.procurarIndice(cpf);
    boolean retorno = false;
    if(i!=-1){
      retorno = true;
    }
    return retorno;
  }

  private int procurarIndice(String cpf){
    Cliente c = null;
    boolean naoAchou = true;
    int indice = -1;

    for(int i = 0; i < this.tamanho && naoAchou; i++){
      c = clientes[i];
      if(c.getCPF().equals(cpf)){
        indice = i;
        naoAchou = false;
      }
    }
    return indice;
  }
}