package banco.contas;

/**
 * Ttulo: PanelCadastro
 * Descrio: Guarda as contas em um array
 *
 * @author Alexandra Barros
 * @version 1.0
 */

public class ArrayContas implements RepositorioContas{

  private ContaAbstrata[] contas;
  private int indiceAtual;
  private int tamanho;

  public ArrayContas() {
    this.tamanho = 10;
    this.indiceAtual = 0;
    this.contas = new ContaAbstrata[tamanho];
  }

  public void inserir(ContaAbstrata conta) throws ContaExistenteException{
    if(!this.existe(conta.getNumero())){
      if(this.indiceAtual == this.tamanho){
        this.tamanho = this.tamanho * 2;
        ContaAbstrata[] temp = new ContaAbstrata[this.tamanho];
        for(int i = 0; i < this.contas.length; i++){
          temp[i] = contas[i];
        }
        this.contas = temp;
      }
      this.contas[this.indiceAtual] = conta;
      this.indiceAtual++;
    }
    else{
      throw new ContaExistenteException("A conta j existe.");
    }
  }

  public void remover(String numero) throws ContaInexistenteException{
    int i = this.procurarIndice(numero);
    if(i!=-1){
      this.contas[i] = this.contas[--this.indiceAtual];
      this.contas[this.indiceAtual] = null;
    }
    else {
      throw new ContaInexistenteException("A conta no existe.");
    }
  }

  public ContaAbstrata procurar(String numero) throws ContaInexistenteException{
    ContaAbstrata c = null;
    int i = this.procurarIndice(numero);
    if(i!=-1){
      c = this.contas[i];
    }
    else {
      throw new ContaInexistenteException("A conta no existe.");
    }
    return c;
  }

  public void atualizar(ContaAbstrata conta) throws ContaInexistenteException{
    int i = this.procurarIndice(conta.getNumero());
    if(i!=-1){
      this.contas[i] = conta;
    }
    else {
      throw new ContaInexistenteException("A conta no existe.");
    }
  }

  public boolean existe(String numero){
    int i = this.procurarIndice(numero);
    boolean retorno = false;
    if(i!=-1){
      retorno = true;
    }
    return retorno;
  }

  public void creditar(String numero, double valor)
      throws ContaInexistenteException{

    int i = this.procurarIndice(numero);
    if(i!=-1){
      this.contas[i].creditar(valor);
    }
    else{
      throw new ContaInexistenteException("A conta no existe.");
    }

  }

  public void debitar(String numero, double valor)
      throws ContaInexistenteException, SaldoInsuficienteException{

    int i = this.procurarIndice(numero);
    if(i!=-1){
      this.contas[i].debitar(valor);
    }
    else{
      throw new ContaInexistenteException("A conta no existe.");
    }

  }

  public void transferir(String origem, String destino, double valor)
      throws ContaInexistenteException, SaldoInsuficienteException{

    int i = this.procurarIndice(origem);
    int j = this.procurarIndice(destino);
    if(i!=-1 && j!=-1){
      this.contas[i].transferir(this.contas[j], valor);
    }
    else{
      throw new ContaInexistenteException("A conta no existe.");
    }

  }

  private int procurarIndice(String numero){
    ContaAbstrata conta = null;
    boolean naoAchou = true;
    int indice = -1;

    for(int i = 0; i < this.tamanho && naoAchou; i++){
      conta = contas[i];
      if(conta!=null && conta.getNumero().equals(numero)){
        indice = i;
        naoAchou = false;
      }
    }
    return indice;
  }
}